# Project Info
# This info is presented in a widget when you share.
# http://framerjs.com/docs/#info.info

Framer.Info =
	title: "Parkeertarieven v6"
	author: "Raymond Uphoff"
	twitter: "@myradon"
	description: "Prototype for City of Amsterdam to replace current implementation"


# Import file "ParkeertarievenToolv0.4FramerExport" (sizes and positions are scaled 1:2)
sketch = Framer.Importer.load "imported/ParkeertarievenToolv0.4FramerExport@2x"

locations = JSON.parse Utils.domLoadDataSync "data/locations-v2.json"
parkingmeters = JSON.parse Utils.domLoadDataSync "data/verkooppunt.json"
tariffareas = JSON.parse Utils.domLoadDataSync "data/tarieven.json"

#import modules
{TextLayer} = require 'TextLayer'

scroll = new ScrollComponent
	width: Screen.width
	height: Screen.height
	backgroundColor: "#fff"

#print Screen.width, Screen.height
scroll.scrollHorizontal = false
#sketch scrollable
sketch.$1_Tablet_.superLayer = scroll.content

##########################################################################
# create maplayer and child div for mapbox
##########################################################################
#mapLayer for mapbox
mapLayer = new Layer
	x: 0
	y: 0
	width: sketch.Maps.width
	height:sketch.Maps.height
	backgroundColor: "#fff"
	propagateEvents: false
	superLayer: sketch.Maps

mapLayer.ignoreEvents = false
mapLayer.html = "<div id='map'></div>"
mapElement = mapLayer.querySelector "#map"
mapElement.style.height = sketch.Maps.height + "px"
mapElement.style.width = sketch.Maps.height + "px"
##########################################################################
# Tariff UI-control
##########################################################################
tariffControl = sketch.tariff_control
tariffControl.index = 101

tariffControl.states =
	hide:
		x: 140
		y: 415
		opacity: 0
		scale: 0.9
		visible: false
	show:
		x: 197
		y: 415
		opacity: 1
		scale: 1

tariffControl.states.animationOptions =
	curve: "ease-in-out"
	time: .25
#imediate switch to stateClosed
tariffControl.stateSwitch("hide")

##########################################################################
# time-day UI-control
##########################################################################
timeDayControl = sketch.time_days_control
timeDayControl.index = 101

timeDayControl.states =
	hide:
		x: 140
		y: 429
		opacity: 0
		scale: 0.9
		visible: false
	show:
		x: 197
		y: 429
		opacity: 1
		scale: 1

timeDayControl.states.animationOptions =
	curve: "ease-in-out"
	time: .25
#imediate switch to stateClosed
timeDayControl.stateSwitch("hide")

facetControls = {
	"btn_tariff": tariffControl
	"btn_time": timeDayControl
}
##########################################################################
# btn-group controls
########################################################################
btnPrev = undefined
facetControl = (btn) ->
	btn.on Events.Tap, (event, btn) ->
		if facetControls[btn.name].states.current.name == "hide"
			facetControls[btn.name].visible = true
			# when previous tapped btn with control and previous tapped btn isn't same as current tapped
			if btnPrev != undefined && btnPrev != btn
				facetControls[btnPrev.name].animate("hide")
		#store tapped btn
		btnPrev = btn
		facetControls[btn.name].stateCycle("hide", "show")

displayMapLayers = (id) ->
		visibility = map.getLayoutProperty id, "visibility"
		if visibility == "visible"
			map.setLayoutProperty id, "visibility", "none"
		else
			map.setLayoutProperty id, "visibility", "visible"

sketch.btn_group.index = 100

for btn in sketch.btn_group.children
	#let's do some stuff
	if btn.name == "btn_garage"
		btn.on Events.Tap, (event, btn) ->
			displayMapLayers("garages")

	if btn.name == "btn_parkingmeter"
		btn.on Events.Tap, (event, btn) ->
			displayMapLayers("parkingmeters")

	if btn.name == "btn_tariff" || btn.name == "btn_time"
		facetControl(btn)
##########################################################################
# search input Maps
########################################################################
sketch._search.index = 100
# Inputfield for search
searchInput = new TextLayer
	#setup: true
	width: 660
	autoSize: true
	contentEditable: true
	parent: sketch._search
	text: "zoek op straatnaam of postcode"
	color: "#999"
	paddingTop: 30
	paddingLeft: 40
	paddingBottom: 30
	paddingRight: 40
	fontSize: 32
	fontStyle: "italic"
	backgroundColor: "transparent"
#store if text is default placeholdertext or user input
searchInput.placeholderText = true

sketch._search.on Events.Tap, (event) ->
	if searchInput.placeholderText == true
		searchInput.text = ""
		searchInput.fontStyle = "normal"
		searchInput.color = "#666"
		searchInput.placeholderText = false

sketch._search.on "keyup", (event) ->
	print searchInput.text
	if event.keyCode is 13
		event.preventDefault()
		target = convertLocation(searchInput.text)
		#let's fecth coordinates of location and when done fly to these coordinates
		getGeoCoordinates(target, setNewLocation)

convertLocation = (target) ->
	#Dutch postal code
	pattern = new RegExp /^[1-9][0-9]{3} ?(?!sa|sd|ss)[a-z]{2}$/i
	if pattern.test(target)
		#print "Ja een postcode!"
		return target
	else
		#Test if input ends with Amsterdam
		pattern = new RegExp /(amsterdam)$/i
		if pattern.test target
			#print "Eindigt met Amsterdam"
			return target
		else
			#print "Eindigt niet met Amsterdam"
			return target + " Amsterdam"

#Fly to new location
setNewLocation = (location) ->
	map.flyTo
		center: location.coordinates
		zoom: 19
		bearing: 5
		speed: 0.75
		curve: 2.5

#GoogleApiKey = "AIzaSyC9fs1w38B6SMxUUdIAS7RfCJnhXzxr4go"
#Do API-call to Google Maps API. It can convert addresses, postalcodes, cities
#into Longitude and lattitude. When Ajax-call is sussesful a calbback function will be executed
getGeoCoordinates = (data, callback) ->
	data = data.replace /\s/g, '+'
	url = "http://maps.google.com/maps/api/geocode/json?address=" + data + "&sensor=false"
	result =
		coordinates: []
		status: undefined

	xhr = new XMLHttpRequest()
	xhr.open "GET", url
	xhr.addEventListener "load", () ->
		if xhr.status == 200
			location = JSON.parse(xhr.responseText)
			#when there are results push geolocations
			if location.results.length
				result.coordinates.push location.results[0].geometry.location.lng, location.results[0].geometry.location.lat
				result.status = location.status
				#Okay we're ready let's eccucute callback-funtion with result
				callback(result)
			else
				print "No results for typed location!"
				result.states = "no results"
	xhr.addEventListener "error", () ->
		print "error retrieving data with Maps-call"
		result.status = "error"
	xhr.send()

##########################################################################
# prep for Mapbox by converting json into geojson
########################################################################
geojsonGaragesArray = []
geojsonSellPointsArray = []
geojsonTariffArray = []
uniqueTarrifs = []
colorTarrifs = {}

#let's transform parkeerlocaties.json into geojson
do ->
	for location in locations.parkeerlocaties
		geolocation = location.parkeerlocatie
		geoPoint =
			"type": "Feature"
			"geometry": JSON.parse(geolocation.Locatie)
			"properties":
				"title": geolocation.title,
				"address": geolocation.adres,
				"postalcode": geolocation.postcode
				"description": geolocation.opmerkingen
				"url": geolocation.url
				"type": geolocation.type

		geojsonGaragesArray.push(geoPoint)

#let's transform parkeermeters.json into geojson
do ->
	for parkingmeter in parkingmeters.features
		geoPoint =
			"type": parkingmeter.type
			"geometry": parkingmeter.geometry
			"properties":
				"title": parkingmeter.properties.VERKOOP_PU
				#"adsress": parkingmeter.properties.OMS
				#"description": parkingmeter.properties.B_DAT_VERK
		#print parkingmeter.properties.VERKOOP_PU
		geojsonSellPointsArray.push(geoPoint)

#let's transform tariffareas.json into geojson
do ->
	areaKeys = Object.keys tariffareas
	for areaKey in areaKeys
		area = tariffareas[areaKey]
		#let's convert into Geojson
		geoPoint =
			"type": "Feature"
			"geometry":
				"type": "MultiPolygon"
				"coordinates": area.location.coordinates
			"properties":
				"title": area.description["0"]
				#amount of different tarriffs per area
				"amount": do() ->
					_num = Object.keys area.tarieven
					_num = _num.length
					#print _num
					return _num
				#tarriffs
				"tarriffs": do() ->
					number = Object.keys area.tarieven
					for i in number
						print areaKey, area.tarieven[i]
						
		geojsonTariffArray.push(geoPoint)

# Mapbox-GL-JS
map = null
initialize = () ->
	#token Raymond Uphoff
	mapboxgl.accessToken = "pk.eyJ1IjoibXlyYWRvbiIsImEiOiJjaXhwdjJ0b2YwMDI1MzJvZnF1dGlncTE5In0.QyMS4Vd8C9klIWw0Rofj4w"
	#define new Map. Fixed overflow problem Markers with CSS
	map = new mapboxgl.Map
		container: mapElement
		style: "mapbox://styles/myradon/cixpv422u005o2rph0gmeawzv"
		#style: "mapbox://styles/mapbox/light-v9"
		center: [4.895168, 52.370216]
		zoom: 12
		pitch: 25

	map.on "style.load", (event) ->
		map.addSource "garages",
			type: "geojson"
			data:
				type: "FeatureCollection"
				features: geojsonGaragesArray
			cluster: true
			clusterMaxZoom: 12 #Max zoom to cluster points on
			clusterRadius: 50 #Radius of each cluster when clustering points (defaults to 50)

		map.addSource "parkingmeters",
			type: "geojson"
			data:
				type: "FeatureCollection"
				features: geojsonSellPointsArray

		map.addSource "tariffareas",
			type: "geojson"
			data:
				type: "FeatureCollection"
				features: geojsonTariffArray

		map.addLayer
			"id": "garages"
			"type": "symbol"
			"source": "garages"
			"layout":
				"text-field": "{title}"
				"text-font": ["Open Sans Semibold"]
				"text-size": 24
				"text-offset": [0, 0.5]
				"text-justify": "left"
				"text-anchor": "left"
# 			"paint":
# 				"text-color": "#ff0000"

		map.addLayer
			"id": "parkingmeters"
			"type": "symbol"
			"source": "parkingmeters"
			"layout":
				"text-field": "{title}"
				"text-font": ["Open Sans Regular"]
				"text-size": 16
				"text-anchor": "center"
# 			"paint":
# 				"text-color": "#0000ff"

		map.addLayer
			id: "tariffareas"
			type: "fill"
			source: "tariffareas"
			paint:
				"fill-color": "#ff0000"
				"fill-opacity": 0.25

# 		#get coordinates of garages from JSON-file
		garageCoordinates = (location) ->
			geo = []
			#Get coordinates string by stripping not applicable characters and split the coordinate-string at the comma
			coordinates = location.parkeerlocatie.Locatie
			.slice 31, -2
			.split ','
			#convert String to Number and push into Array
			geo.push Number coordinates[0]
			geo.push Number coordinates[1]
			return geo

		for location, i in locations.parkeerlocaties
			#create a DOM element for the marker. Marker is styled with CSS
			el = document.createElement "div"
			el.className = "garage-icon"
			el.id = "garage-icon-" + i
			#print el.id

			el.addEventListener "tap", () ->
				print "icon clicked class: " + @classList + ", with id:" + @id

			#add marker to map
			coordinates = garageCoordinates(location)
			new mapboxgl.Marker el , {offset: [-70, -20]}
			.setLngLat coordinates
			.addTo map

Utils.delay 0.1, -> initialize() # make sure map is ready
